
#!/bin/bash
#
# -- Author : François Massonnet, francois.massonnet@ic3.cat
# -- Date   : 30 Jan 2015
# -- At     : IC3, Barcelona
# -- Modified : 19 Jan 2016 - omar.bellprat@bsc.es
#               12 Feb 2020 - change library ncdf to ncdf4 - Yohan Ruprich-Robert, yohan.ruprich@bsc.es 
#               29 Jul 2020 - specify R version R/3.6.1 - Oriol Tintó, oriol.tinto@bsc.es
#               1 Sept 2020 - better documentation and flexible loop_modif_restart.sh - etienne.tourigny@bsc.es
#
# -- Purpose: Generation of an arbitrary number of NEMO oceanic restarts that are copies of a reference, plus a perturbation
#
# -- Method : The reference file is duplicated in this script, then read by a R script. The perturbation is introduced and finally written to this latter file.
#             The script must be placed into the restart directory (e.g. NEMO_Restart_23). The generated restarts have to be renamed after generation. This 
#             script has been tested on MareNostrum3.
#
# -- Input  : NEMO ocean restart from an EC-Earth 3.x run
# -- Output : N restarts with the same name,but with an index fc0, fc1, ... fcN-1 appended
#
# -- Limitations: Only the surface conditions are perturbed (level index: 1) but this can be changed in the R script

# load the appropriate R version before calling the script (this is done in loop_modif_restart.sh)
# on MN4 that can be done with "module load R/3.6.1"
# but this might work on other systems e.g. nord3 "module load R"

set -o errexit
set -o nounset
set -x

if [ $# == 0 ] ; then
  echo "gener_perturbation.bash ocean_restart_file Nmembers"
  exit
fi

filein=$1
nmemb=$2

# ---------------------------------------------------------
var=tn          # Variable to be perturbed
#per=0.0001      # Standard deviation of gaussian perturbation to be applied,
per=0.0002      # in units of the variable (for tn: in K for example)

for jmemb in `seq 0 $(( $nmemb -1 ))`
do
  echo $jmemb
  # 1. Make a copy of the original file, with the new name
  filenew="${filein%.nc}_fc${jmemb}.nc"
# filenew="${filein%.nc}.nc"
  cp $filein ${filein}.backup
  cp $filein $filenew

  # 2. Prepare the R script
  echo "#!/usr/bin/env Rscript
  library(ncdf4)

  # François Massonnet, 30 Jan 2015
  # Adds a gaussian perturbation at the first level of a 3D field
  # Tested only for NEMO restarts
  #
  # This script should be called by a bash script so that the variable and file names are specified, as well as the perturbation

  varname='$var'
  filein <- '$filenew'
  ex.nc           <- nc_open(filein,write=TRUE)
  spert <- $per

  myvar     <- ncvar_get(ex.nc, varname)
  myvarpert <- myvar
  for (i in seq(1,dim(myvar)[1])){
    for (j in seq(1,dim(myvar)[2])){
      if (myvar[i,j,1] != 0){
        myvarpert[i,j,1] = myvarpert[i,j,1] + rnorm(1,sd=spert)
      }
    }
  }

  ncvar_put(ex.nc,varname,myvarpert)
  nc_close(ex.nc)" > Rtmpfile.R

  chmod 744 Rtmpfile.R

  # 3. Run the R script, that produces the new NetCDF
  ./Rtmpfile.R
done


